<?php

// IMAP Sorting Constant
/** @constant integer SORTTHREAD  Sort By Thread. */
@define('SORTTHREAD', 161);

// MIMP internal indexing strings
define('MIMP_MSG_SEP', "\0");

/**
 * MIMP Base Class.
 *
 * $Horde: mimp/lib/MIMP.php,v 1.51 2005/01/03 14:35:16 jan Exp $
 *
 * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
 * Copyright 1999-2005 Jon Parise <jon@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @version $Revision: 1.51 $
 * @since   MIMP 0.0.1
 * @package MIMP
 */
class MIMP {

    /**
     * Take a Horde_Mobile_card and add global MIMP menu items.
     *
     * @access public
     *
     * @param object Horde_Mobile_linkset &$menu  The menu linkset, with
     *                                            page-specific options
     *                                            already filled in.
     */
    function addMIMPMenu(&$menu)
    {
        $items = array();
        if (strpos($_SERVER['PHP_SELF'], 'mailbox.php') === false ||
            (MIMP::getThisMailbox() != 'INBOX')) {
            $items[Util::addParameter(Horde::applicationUrl('mailbox.php'), 'mailbox', 'INBOX')] = 'INBOX';
        }
        if (strpos($_SERVER['PHP_SELF'], 'compose.php') === false) {
            $items[Util::addParameter(Horde::applicationUrl('compose.php'), 'u', base_convert(microtime() . mt_rand(), 10, 36))] = _("Compose");
        }
        if (strpos($_SERVER['PHP_SELF'], 'folders.php') === false) {
            $items[Horde::applicationUrl('folders.php')] = _("Folders");
        }
        // @TODO
        // if ($options_link = Horde::getServiceLink('options', 'mimp')) {
        //     $items[Util::addParameter($options_link, 'mobile', 1, false)] = _("Options");
        // }
        $items[Auth::addLogoutParameters(Horde::applicationUrl('login.php'), AUTH_REASON_LOGOUT)] = _("Log out");

        foreach ($items as $link => $label) {
            $menu->add(new Horde_Mobile_link($label, $link, $label));
        }
    }

    /**
     * Determines if the given mail server is the "preferred" mail server for
     * this web server.  This decision is based on the global 'SERVER_NAME'
     * and 'HTTP_HOST' server variables and the contents of the 'preferred'
     * either field in the server's definition.  The 'preferred' field may
     * take a single value or an array of multiple values.
     *
     * @param string $server      A complete server entry from the $servers
     *                            hash.
     * @param optional TODO $key  TODO
     *
     * @return boolean  True if this entry is "preferred".
     */
    function isPreferredServer($server, $key = null)
    {
        static $urlServer;

        if (!isset($urlServer)) {
            $urlServer = Util::getFormData('server');
        }

        if (!empty($urlServer)) {
            return $key == $urlServer;
        }

        if (!empty($server['preferred'])) {
            if (is_array($server['preferred'])) {
                foreach ($server['preferred'] as $preferred) {
                    if ($preferred == $_SERVER['SERVER_NAME'] ||
                        $preferred == $_SERVER['HTTP_HOST']) {
                        return true;
                    }
                }
            } elseif ($server['preferred'] == $_SERVER['SERVER_NAME'] ||
                      $server['preferred'] == $_SERVER['HTTP_HOST']) {
                return true;
            }
        }

        return false;
    }

    /**
     * Tack on any prefixes that need to be put at the beginning of
     * the folder path.
     *
     * @return string  The prefix, if any.
     */
    function preambleString()
    {
        return $_SESSION['mimp']['folders'] . $_SESSION['mimp']['namespace'];
    }

    /**
     * Tack on any prefixes that need to be put at the beginning of
     * the folder path, but don't add it to INBOX, and return the full
     * mailbox name.
     *
     * @param string $mailbox   The folder path that needs the prefixes to be
     *                          prepended.
     *
     * @return string  The full folder path with prefixes if needed.
     */
    function addPreambleString($mailbox)
    {
        if (empty($mailbox) || $mailbox == 'INBOX') {
            return $mailbox;
        }
        return MIMP::preambleString() . $mailbox;
    }

    /**
     * Generate a full c-client server specification string.
     *
     * @access public
     *
     * @param optional string $protocol  Override the protocol currently being
     *                                   used.
     *
     * @return string  The full spec string.
     */
    function serverString($protocol = null)
    {
        $srvstr = '{' . $_SESSION['mimp']['server'];

        /* If port is not specified, don't include it in Server String. */
        if (isset($_SESSION['mimp']['port'])) {
            $srvstr .= ':' . $_SESSION['mimp']['port'];
        }

        if (!isset($protocol)) {
            $protocol = $_SESSION['mimp']['protocol'];
        }

        /* If protocol is not specified, don't include it in Server String. */
        if (!empty($protocol)) {
            $srvstr .= '/' . $protocol;
        }

        $srvstr .= '}';
        return $srvstr;
    }

    /**
     * Get the plain text label that is displayed for the current
     * mailbox, replacing IMP_SEARCH_MBOX with an appropriate string and
     * removing namespace and folder prefix information from what is
     * shown to the user.
     *
     * @access public
     *
     * @return string  The plain text label.
     */
    function getLabel()
    {
        $label = '';

        if ($_SESSION['mimp']['mailbox'] == 'INBOX') {
            $label = _("Inbox");
        } else {
            $label = String::convertCharset($_SESSION['mimp']['mailbox'], 'UTF7-IMAP');
            if (strcmp($_SESSION['mimp']['folders'], String::substr($label, 0, String::length($_SESSION['mimp']['folders']))) == 0) {
                $label = String::substr($label, String::length($_SESSION['mimp']['folders']));
            }
            if (strcmp($_SESSION['mimp']['namespace'], String::substr($label, 0, String::length($_SESSION['mimp']['namespace']))) == 0) {
                $label = String::substr($label, String::length($_SESSION['mimp']['namespace']));
            }
        }

        return $label;
    }

    /**
     * Returns the bare address.
     *
     * @access public
     *
     * @param string $address             The address string.
     * @param optional boolean $multiple  Should we return multiple results?
     *
     * @return mixed  See MIME::bareAddress().
     */
    function bareAddress($address, $multiple = false)
    {
        require_once 'Horde/MIME.php';
        return MIME::bareAddress($address, $_SESSION['mimp']['maildomain'], $multiple);
    }

    /**
     * Use the Registry to expand names and returning error information for
     * any address that is either not valid or fails to expand.
     *
     * @param string $addrString  The name(s) or address(es) to expand.
     * @param bool $full          If true generate a full, rfc822-valid
     *                            address list.
     *
     * @return mixed   Either a string containing all expanded addresses or
     *                 an array containing all matching address or an error
     *                 object.
     */
    function expandAddresses($addrString, $full = false)
    {
        if (!preg_match('|[^\s]|', $addrString)) {
            return '';
        }

        global $prefs, $registry;

        require_once 'Mail/RFC822.php';
        require_once 'Horde/MIME.php';

        $addrString = preg_replace('/,\s+/', ',', $addrString);
        $parser = &new Mail_RFC822(null, '@INVALID');
        $search_fields = array();

        $src = explode("\t", $prefs->getValue('search_sources'));
        if ((count($src) == 1) && empty($src[0])) {
            $src = array();
        }

        if (($val = $prefs->getValue('search_fields'))) {
            $field_arr = explode("\n", $val);
            foreach ($field_arr as $field) {
                $field = trim($field);
                if (!empty($field)) {
                    $tmp = explode("\t", $field);
                    if (count($tmp) > 1) {
                        $source = array_splice($tmp, 0, 1);
                        $search_fields[$source[0]] = $tmp;
                    }
                }
            }
        }

        $arr = MIME::rfc822Explode($addrString, ',');
        $results = $registry->call('contacts/search', array($arr, $src, $search_fields));
        if (is_a($results, 'PEAR_Error')) {
            return $results;
        }

        $ambiguous = false;
        $error = false;
        $i = 0;
        $missing = array();

        foreach ($results as $res) {
            $tmp = $arr[$i];
            if ($parser->validateMailbox(MIME::encodeAddress($tmp, null, ''))) {
                // noop
            } elseif (count($res) == 1) {
                if ($full) {
                    if (strpos($res[0]['email'], ',') !== false) {
                        $arr[$i] = $res[0]['name'] . ': ' . $res[0]['email'] . ';';
                    } else {
                        list($mbox, $host) = explode('@', $res[0]['email']);
                        $arr[$i] = MIME::rfc822WriteAddress($mbox, $host, $res[0]['name']);
                    }
                } else {
                    $arr[$i] = $res[0]['name'];
                }
            } elseif (count($res) > 1) {
                /* Handle the multiple case - we return an array with all found
                   addresses. */
                $arr[$i] = array($arr[$i]);
                foreach ($res as $one_res) {
                    if ($full) {
                        if (strpos($one_res['email'], ',') !== false) {
                            $arr[$i][] = $one_res['name'] . ': ' . $one_res['email'] . ';';
                        } else {
                            $mbox_host = explode('@', $one_res['email']);
                            if (isset($mbox_host[1])) {
                                $arr[$i][] = MIME::rfc822WriteAddress($mbox_host[0], $mbox_host[1], $one_res['name']);
                            }
                        }
                    } else {
                        $arr[$i][] = $one_res['name'];
                    }
                }
                $ambiguous = true;
            } else {
                /* Handle the missing/invalid case - we should return error info
                   on each address that couldn't be expanded/validated. */
                $error = true;
                if (!$ambiguous) {
                    $arr[$i] = PEAR::raiseError(null, null, null, null, $arr[$i]);
                    $missing[$i] = $arr[$i];
                }
            }
            $i++;
        }

        if ($ambiguous) {
            foreach ($missing as $i => $addr) {
                $arr[$i] = $addr->getUserInfo();
            }
            return $arr;
        } elseif ($error) {
            return PEAR::raiseError(_("Please resolve ambiguous or invalid addresses."), null, null, null, $res);
        } else {
            return implode(', ', $arr);
        }
    }

    /**
     * Returns the current mailbox.
     *
     * @access public
     *
     * @return string  The current mailbox name.
     */
    function getThisMailbox()
    {
        return $_SESSION['mimp']['thismailbox'];
    }

    /**
     * Returns the appropriate link to call the message composition
     * screen.
     *
     * @access public
     *
     * @param optional mixed $args   List of arguments to pass to compose.php.
     *                               If this is passed in as a string, it will
     *                               be parsed as a
     *                               toaddress?subject=foo&cc=ccaddress
     *                               (mailto-style) string.
     * @param optional array $extra  Hash of extra, non-standard arguments to
     *                               pass to compose.php.
     *
     * @return string  The link to the message composition screen.
     */
    function composeLink($args = array(), $extra = array())
    {
        /* Make sure the compose window always knows which mailbox
           it's in, for replying, forwarding, marking as answered,
           etc. */
        if (!isset($extra['thismailbox'])) {
            $extra['thismailbox'] = MIMP::getThisMailbox((isset($extra['array_index'])) ? $extra['array_index'] : null);
        }

        if (is_string($args)) {
            $string = $args;
            $args = array();
            if (($pos = strpos($string, '?')) !== false) {
                parse_str(substr($string, $pos + 1), $args);
                $args['to'] = substr($string, 0, $pos);
            } else {
                $args['to'] = $string;
            }
        }

        /* Merge the two argument arrays. */
        $args = array_merge($args, $extra);

        /* Convert the $args hash into proper URL parameters. */
        $args_list = array();
        foreach ($args as $key => $val) {
            if (!empty($val) || is_int($val)) {
                $args_list[$key] = $val;
            }
        }

        return Util::addParameter(Horde::applicationUrl('compose.php'), $args_list);
    }

    /**
     * Generate an URL to the logout screen that includes any known
     * information, such as username, server, etc., that can be filled
     * in on the login form.
     *
     * @access public
     *
     * @return string  Logout URL with logout parameters added.
     */
    function logoutUrl($uri = 'login.php', $reason = null)
    {
        $params = array(
            'imapuser' => isset($_SESSION['mimp']['user']) ? $_SESSION['mimp']['user'] :
                                                             Util::getFormData('imapuser'),
            'server'   => isset($_SESSION['mimp']['server']) ? $_SESSION['mimp']['server'] :
                                                               Util::getFormData('server'),
            'port'     => isset($_SESSION['mimp']['port']) ? $_SESSION['mimp']['port'] :
                                                             Util::getFormData('port'),
            'protocol' => isset($_SESSION['mimp']['protocol']) ? $_SESSION['mimp']['protocol'] :
                                                                 Util::getFormData('protocol'),
            'folders'  => isset($_SESSION['mimp']['folders']) ? $_SESSION['mimp']['folders'] :
                                                                Util::getFormData('folders'),
            'language' => isset($_SESSION['mimp']['language']) ? $_SESSION['mimp']['language'] :
                                                                 Util::getFormData('language')
        );

        foreach ($params as $key => $val) {
            if (!empty($val)) {
                $uri = Util::addParameter($uri, $key, $val);
            }
        }

        return Horde::applicationUrl($uri, true);
    }

    /**
     * If there is information available to tell us about a prefix in
     * front of mailbox names that shouldn't be displayed to the user,
     * then use it to strip that prefix out.
     *
     * @param string $folder            The folder name to display.
     * @param optional boolean $decode  Whether or not the folder needs to be
     *                                  decoded from UTF7-IMAP.
     *
     * @return string  The folder, with any prefix gone.
     */
    function displayFolder($folder, $decode = true)
    {
        $prefix = MIMP::preambleString();
        if (substr($folder, 0, strlen($prefix)) == $prefix) {
            $folder = substr($folder, strlen($prefix));
        }

        return $decode ? String::convertCharset($folder, 'UTF7-IMAP') : $folder;
    }

    /**
     * Make sure the user has been authenticated to view the page.
     *
     * @access public
     *
     * @param optional mixed $flags     Any flags to pass to imap_open().
     *                                  See Auth_mimp::authenticate().
     *                                  However, if this is the string
     *                                  'horde', we just check for Horde auth
     *                                  and don't bother the IMAP server.
     * @param optional boolean $return  If this is true, return false instead
     *                                  of exiting/redirecting if
     *                                  authentication fails.
     *
     * @return boolean  True on success, false on error.
     */
    function checkAuthentication($flags = 0, $return = false)
    {
        if ($flags === 'horde') {
            $reason = Auth::isAuthenticated();
        } else {
            $auth_mimp = &Auth::singleton(array('mimp', 'mimp'));
            $auth_mimp->authenticateOptions(array('flags' => $flags));
            $reason = $auth_mimp->authenticate();
        }

        if ($reason !== true) {
            if ($return) {
                return false;
            }

            if (Util::getFormData('popup')) {
                Util::closeWindowJS();
            } else {
                $url = Auth::addLogoutParameters(MIMP::logoutUrl());
                $url = Util::addParameter($url, 'url', Horde::selfUrl(true), false);
                header('Location: ' . $url);
            }
            exit;
        }

        return true;
    }

    /**
     * Filter a string, if requested.
     *
     * @access public
     *
     * @param string $text  The text to filter.
     *
     * @return string  The filtered text (if requested).
     */
    function filterText($text)
    {
        global $conf, $prefs;

        if ($prefs->getValue('filtering')) {
            require_once 'Horde/Text/Filter.php';
            $text = Text_Filter::filter($text, 'words', array('words_file' => $conf['msg']['filtering']['words'], 'replacement' => $conf['msg']['filtering']['replacement']));
        }

        return $text;
    }

    /**
     * Get the initial URL.
     *
     * @access public
     *
     * @param optional string $actionID  The action ID to perform on the
     *                                   initial page.
     *
     * @return string  The initial URL.
     */
    function getInitialUrl($actionID = null)
    {
        $init_url = $GLOBALS['prefs']->getValue('initial_page');
        if ($init_url == 'folders.php') {
            $url = Horde::applicationUrl($init_url, true);
        } else {
            $url = Horde::applicationUrl('mailbox.php', true);
            $url = Util::addParameter($url, 'mailbox', MIMP::addPreambleString($init_url), false);
        }

        if (!empty($actionID)) {
            $url = Util::addParameter($url, 'actionID', $actionID, false);
        }

        return $url;
    }

}
